import scrapy, copy, six, logging, codecs, urllib
from urlparse import urlparse
from selenium import webdriver
from collections import defaultdict
from scrapy import log
from scrapy.utils.python import to_bytes
from scrapy.http import Request, HtmlResponse
from scrapy.spiders import CrawlSpider, Rule
from scrapy.linkextractors import LinkExtractor
from bs4 import BeautifulSoup

def recompose_url(href_parts):
   if href_parts[1].startswith("http"):
      return href_parts[1]
   elif href_parts[1].startswith("/"):
      parent_url = urlparse(href_parts[0])
      return "%s://%s%s" % (parent_url.scheme, parent_url.netloc, href_parts[1])
   else:
      return ''.join(href_parts)

def process_hrefs(href):
   href = href.split(";jsessionid=")[0].split("?timestamp=")[0].split(";JSESSIONID")[0]
   if " " in href: 
      href_parts = href.split()
      if len(href_parts) > 1:
         return recompose_url(href_parts)
   elif "%20" in href:
      href_parts = href.split("%20")
      if len(href_parts) > 1:
         return recompose_url(href_parts)
   return href

class HealthSpider(CrawlSpider):
   name = "health"
   allowed_domains = []
   start_urls = []
   rules = []
   keywords = defaultdict(lambda : defaultdict(set))
   countries = defaultdict(set)
   organizations = defaultdict(lambda : defaultdict(set))
   
   def __init__(self, *args, **kwargs):
      logging.getLogger("selenium").setLevel(logging.INFO)
      self.organization_id = int(kwargs["org"])
      self._load_keywords()
      self._load_countries()
      self._load_domains()
      self._make_rules()
      super(HealthSpider, self).__init__(*args, **kwargs)

   def _load_keywords(self):
      with open("keywords.csv", 'r') as f:
         for line in f:
            fields = line.lower().strip().split('\t')
            self.keywords[int(fields[0])] = fields[1]

   def _load_countries(self):
      with open("countries.csv", 'r') as f:
         for line in f:
            fields = line.lower().strip().split('\t')
            self.countries[int(fields[0])] = set(fields[1:])
   
   def _load_domains(self):
      with codecs.open("organizations.csv", 'r', encoding = "utf-8") as f:
         for line in f:
            fields = line.strip().split('\t')
            org_id = int(fields[0])
            self.organizations[org_id]["full_name"] = fields[1].lower().strip()
            if fields[2].strip() != "":
               self.organizations[org_id]["aliases"].add(fields[2].strip())
            self.organizations[org_id]["urls"] = set([str(x.strip()) for x in fields[3].strip().split(';')])
            self.organizations[org_id]["tlds"] = set([urlparse(str(x.strip())).netloc for x in fields[3].strip().split(';')])
            if len(fields) > 4:
               self.organizations[org_id]["aliases"] |= set([x.strip() for x in fields[4].strip().split(';') if x.strip() != ""])

   def _make_rules(self):
      for url in self.organizations[self.organization_id]["urls"]:
         parsed_uri = urlparse(url)
         path_rule = parsed_uri.path.encode("string-escape").split('/')
         path_rule.pop()
         path_rule = "%s/" % '/'.join(path_rule)
         self.start_urls.append(url)
         self.allowed_domains.append(parsed_uri.netloc.replace("www.", ""))
         self.rules.append(Rule(LinkExtractor(allow = path_rule,
                                              deny = ("calendar", "listevents", "search", "lang/en/home/projets_afd/villes/wuf/", "public/public/public", "cHash", 
                                                      "default.aspx/default.aspx", "facetcriteria", "lang--es", "lang--fr", "Search", "lang=es", "lang=fr",
                                                      "lang=ru", "lang=zh", "lang=ar", "language=es", "language=fr", "language=de", "language=it", "ADSSChck",
                                                      "/pages/pages/", "/es/", "/ru/", "/fr/", "/ar/", "/zh/", "locale=ar", "locale=zh", "locale=ru", "locale=fr",
                                                      "locale=es", "/galleries/galleries/", "/node/node/", "amp=&amp", "video", "srch-pg", "Search", "arabic",
                                                      "mail", "photo", "spanish", "french", "audio", "download", "chinese", "russian", "media", "/pressroom/reactions/",
                                                      "field_tags", "field_relate_experts", "sm_field", "im_field", "sm_document", "/tags/", "=field_", "=subjects"),
                                              process_value = process_hrefs),
                                              callback = "parse_item", follow = True))

   def parse_item(self, response):
      self.driver = webdriver.PhantomJS(executable_path = "/home/mikk/Documents/wb_crawl/phantomjs",
                                        service_args = ["--webdriver-loglevel=NONE"],
                                        service_log_path = "/dev/null")
      self.driver.set_page_load_timeout(30)
      self.driver.get(response.request.url)
      body = BeautifulSoup(self.driver.page_source, "lxml")
      [s.extract() for s in body("script")]
      page_text = body.find("body").getText(separator = " ")
      page_text_lower = page_text.lower()
      page_data = {}
      parsed_uri = urlparse(response.request.url)
      page_data["organization_id"] = self.organization_id
      page_data["domain"] = parsed_uri.netloc
      page_data["page"] = parsed_uri.path
      page_data["keywords"] = {}
      page_data["countries"] = {}
      page_data["organizations"] = {}
      page_data["organizations_links"] = defaultdict(int)
      for keyword in self.keywords:
         occurrences = page_text_lower.count(self.keywords[keyword])
         if occurrences > 0:
            page_data["keywords"][keyword] = occurrences
      for country in self.countries:
         occurrences = page_text_lower.count(self.keywords[keyword])
         for alias in self.countries[country]:
            occurrences += page_text_lower.count(alias)
         if occurrences > 0:
            page_data["countries"][country] = occurrences
      for org in self.organizations:
         if org != self.organization_id:
            occurrences = page_text_lower.count(self.organizations[org]["full_name"])
            for alias in self.organizations[org]["aliases"]:
               occurrences += page_text.count(alias)
            if occurrences > 0:
               page_data["organizations"][org] = occurrences
      for anchor in body.find_all("a"):
         if anchor.has_attr("href"):
            url = urlparse(anchor["href"])
            for org in self.organizations:
               if org != self.organization_id and url.netloc in self.organizations[org]["tlds"]:
                  page_data["organizations_links"][org] += 1
      if len(page_data["keywords"]) > 0:
         log.msg("Pages in queue: %s" % len(self.crawler.engine.slot.scheduler), level = log.INFO)
         yield page_data
      self.driver.quit()
